#!/usr/local/bin/perl

# Conrol Supermicro X9 fan mode in response to HDD temperature.
# Fan controlled = FANA
# Customized for remote IPMI
# Cron Job command = perl /mnt/Tank1/System/Scripts/fancontrol/fancontrol.pl (every minute)
# Logfile = /mnt/Tank1/System/Scripts/fancontrol/HD_TempLog.txt

# edit the following values
$number_of_hard_drives = 16;
$hd_designator = "/dev/da";
$min_fan_speed = 600;
$max_fan_speed = 7000;

$LogFile = '/mnt/Tank1/System/Scripts/fancontrol/HD_TempLog.txt';
$min_fan_speed *= 1.3;
$max_fan_speed *= 0.8;

# edit BMC IP address and username
$ipmitool = "/usr/local/bin/ipmitool -I lanplus -H <IP> -U <USER> -f /root/.ipmi ";

# edit nothing below this line

use POSIX qw(strftime);
$datestring = strftime "%F %H:%M:%S", localtime;

open (LOGFILE, ">>$LogFile");

$max_temp = 0;

foreach $item (0..$number_of_hard_drives-1) {
  $command = "/usr/local/sbin/smartctl -A $hd_designator$item | grep Temp";

  # print "$command\n";

  $output = `$command`;
  @vals = split(" ", $output);

  # grab item 10 from the output, which is the hard drive temperature
  $temp = "$vals[9]\n";

  # update maximum drive temperature
  $max_temp = $temp if $temp > $max_temp;
}

if ($max_temp > 39){
  `$ipmitool raw 0x30 0x45 0x01 0x01`
  # at least one hard drive is 40°C or higher
  # set fan speed control to Full
  }

elsif ($max_temp > 37){
  # at least one hard drive is 38°C or higher
  # set fan speed to Heavy IO
 `$ipmitool raw 0x30 0x45 0x01 0x04`
}

else {
  # all hard drive temperatures are 37°C or cooler
  # set fan speed control to Optimal
  `$ipmitool raw 0x30 0x45 0x01 0x02`
}

# reset BMC if temps are cool, and fan is not slow
if ($max_temp < 36){
  $command = "$ipmitool sdr | grep FANA";
  $output = `$command`;
  @vals = split(" ", $output);
  $fan_speed = "$vals[2]";

  if ($fan_speed > $min_fan_speed){
    `$ipmitool bmc reset cold`;
  }
}

# reset BMC if temps are warm, and fan is not fast
if ($max_temp > 40){
  $command = "$ipmitool sdr | grep FANA";
  $output = `$command`;
  @vals = split(" ", $output);
  $fan_speed = "$vals[2]";

  if ($fan_speed < $max_fan_speed){
    `$ipmitool bmc reset cold`;
  }
}

print LOGFILE "$datestring - $max_temp";
close (LOGFILE);